@@ -141,21 +141,6 @@ NULL
   UseMethod(generic = '.MARGIN', object = x)
 }
 
-#' Combine and Select Features
-#'
-#' @template param-dots-method
-#' @param object An object
-#'
-#' @return A vector of features selected
-#'
-#' @keywords internal
-#'
-#' @export .SelectFeatures
-#'
-.SelectFeatures <- function(object, ...) {
-  UseMethod(generic = '.SelectFeatures', object = object)
-}
-
 #' Add in metadata associated with either cells or features.
 #'
 #' Adds additional data to the object. Can be any piece of information
@@ -481,6 +466,8 @@ CreateMolecules <- function(coords, ...) {
 #' Create a \code{\link[SeuratObject:Segmentation-class]{Segmentation}} Objects
 #'
 #' @param coords The coordinates of cell segmentations
+#' @param compact Logical indicating whether or not the object should only store segmentation data
+#' in the \code{sf.data} slot; see \link{Segmentation-class} for details.
 #'
 #' @return A \code{\link[SeuratObject:Segmentation-class]{Segmentation}} object
 #'
@@ -488,7 +475,7 @@ CreateMolecules <- function(coords, ...) {
 #'
 #' @concept spatial
 #'
-CreateSegmentation <- function(coords) {
+CreateSegmentation <- function(coords, compact = FALSE) {
   UseMethod(generic = 'CreateSegmentation', object = coords)
 }
 
@@ -816,6 +803,11 @@ GetImage <- function(object, mode = c('grob', 'raster', 'plotly', 'raw'), ...) {
 }
 
 #' Get tissue coordinates
+#' 
+#' Retrieve tissue coordinates from spatial objects. 
+#'
+#' Spatial classes may have specific implementations; 
+#' refer to those documentation pages for more details.
 #'
 #' @template param-dots-method
 #' @param object An object
